package tree;
import java.util.*;

/**
 * Represents the empty state of a Tree.
 * Uses the Singleton pattern.
 */
public class Empty extends ATreeNode {
    public static Empty Singleton = new Empty();
    private Empty() {
    }

    Object getRoot(Tree owner) {
        throw new NoSuchElementException("Empty.getRoot(): empty tree has no data.");
    }

    void setRoot(Tree owner, Object dat) {
        throw new NoSuchElementException("Empty.setRoot(): empty tree has no data.");
    }

    /**
     * After insertion of the data object, the owner Tree becomes non-empty containg 
     * the inserted data object at the root.
     */
    void insertRoot(Tree owner, Object dat) {
        owner.setRootNode(new NonEmpty(dat));
    }

    /**
     * Cannot remove root from an empty tree.
     */
    void remRoot(Tree owner) {
        throw new NoSuchElementException("Empty.remRoot(): empty tree has no data.");
    }

    Tree get_leftMostChild(Tree owner) {
        throw new NoSuchElementException("Empty.get_leftMostChild(): empty tree has no subtree.");
    }

    void set_leftMostChild(Tree owner, Tree lmc) {
        throw new NoSuchElementException("Empty.set_leftMostChild(): empty tree has no subtree.");
    }

    Tree get_rightSibling(Tree owner) {
        throw new NoSuchElementException("Empty.get_rightSibling(): empty tree has no right sibling.");
    }

    void set_rightSibling(Tree owner, Tree rs) {
        throw new NoSuchElementException("Empty.set_rightSibling(): empty tree has no right sibling.");
    }
       
    Object execute(Tree host, ITreeVis v, Object... inp) {
        return v.emptyCase(host, inp);
    }
}

