package tree;

/**
 * Represents the non-empty state of the Tree.
 */
class NonEmpty extends ATreeNode {
    private Object _dat;  // the data object
    private Tree _leftMostChild;  // the leftmost subtree.
    private Tree _rightSibling;  // the sibling immediately to the right of this Tree.
    
    NonEmpty(Object dat, Tree lmc, Tree rs) {
        _dat = dat;
        _leftMostChild = lmc;
        _rightSibling = rs;
    }
    
    /**
     * Initializes this node to contain a data object, an empty leftmost child
     * and an empty right sibling.
     * This means this node is a "leaf" node.
     */
    NonEmpty(Object dat) {
        this(dat, new Tree(), new Tree());
    }
    
    
    Object getRoot(Tree owner) {
        return _dat;
    }
    
    void setRoot(Tree owner,Object dat) {
        _dat = dat;
    }
    
    /**
     * Cannot insert data at the root of a non-empty Tree.
     */
    void insertRoot(Tree owner, Object dat) {
        throw new IllegalStateException("NonEmpty.insertRoot(" + dat + ") is not allowed.");
    }
    
    /**
     * Problem 4.c
     * Remove this node only if the left most child is empty or when the right 
     * sibling is empty.
     * When the leftmost child is empty, this node is replaced by its right sibling's node.
     * When the right sibling is empty, this node is replaced by its leftmost child's node.
     * Throw an IllegalStateException otherwise.
     * STUDENTS TO COMPLETE
     */
    void remRoot(final Tree owner) {
        _leftMostChild.execute(new ITreeVis() {
            public Object emptyCase(Tree h, Object... nu) {
                owner.setRootNode(_rightSibling.getRootNode());
                return null;
            }
            
            public Object nonMTCase(Tree h, Object... nu) {
                return _rightSibling.execute(new ITreeVis() {
                    public Object emptyCase(Tree h1, Object... nu) {
                        owner.setRootNode(_leftMostChild.getRootNode());
                        return null;
                    }                
                    public Object nonMTCase(Tree h, Object... nu) {                    
                        throw new IllegalStateException("NonEmpty.remRoot(): left most child and right sibling are not empty.");
                    }
                });
            }
        });
    }
            
            Tree get_leftMostChild(Tree owner) {
                return(_leftMostChild);
            }
            
            void set_leftMostChild(Tree owner, Tree lmc) {
                _leftMostChild = lmc;
            }
            
            Tree get_rightSibling(Tree owner) {
                return _rightSibling;
            }
            
            void set_rightSibling(Tree owner, Tree rs) {
                _rightSibling = rs;
            }
            
            Object execute(Tree host, ITreeVis v, Object... inp) {
                return v.nonMTCase(host, inp);
            }
        }
                               
