package tree;

/**
 * An n-ary tree with arbitrary number of subtrees can be implemented using
 * the "leftmost child/right sibling" representation.
 */
public class Tree {
    private ATreeNode _rootNode;  // state pattern
    
    public Tree() {
        _rootNode = Empty.Singleton;  // a new Tree is always empty.
    }
    
    public Object getRoot() {
        return _rootNode.getRoot(this);
    }
    
    public void setRoot(Object dat) {
        _rootNode.setRoot(this, dat);
    }
    
    /**
     * Insert root data only when the Tree is empty;
     * Throw an IllegalStateException otherwise;
     */
    public void insertRoot(Object dat) {
        _rootNode.insertRoot(this, dat);
    }
    
    /**
     * Remove the root only if the left most child is empty or when the right 
     * sibling is empty.  
     * When the leftmost child is empty, this tree mutates to become its right sibling.
     * When the right sibling is empty, this tree mutates to become its leftmost child.
     * Throw an IllegalStateException otherwise.
     */
    public void remRoot() {
        _rootNode.remRoot(this);
    }
    
    /**
     * Returns the leftmost child.
     */
    public Tree get_leftMostChild() {
        return _rootNode.get_leftMostChild(this);
    }
    
    /**
     * Sets the leftmost child to another Tree.
     */
    public void set_leftMostChild(Tree lmc) {
        _rootNode.set_leftMostChild(this, lmc);
    }
    
    /**
     * Returns the right sibling.
     */
    public Tree get_rightSibling() {
        return _rootNode.get_rightSibling(this);
    }
    
    /**
     * Sets the right sibling to another Tree.
     */
    public void set_rightSibling(Tree rs) {
        _rootNode.set_rightSibling(this, rs);
    }

    /**
     * Performs a visitor algorithm on this Tree with a given input.
     */
    public Object execute(ITreeVis v, Object... inp) {
        return _rootNode.execute(this, v, inp);
    }

    // Package private methods to set and get the state of this Tree.
    
    /**
     * Sets the root node to another node.
     */
    void setRootNode(ATreeNode n) {
        _rootNode = n;
    }
    
    /**
     * Returns the root node.
     */
    ATreeNode getRootNode() {
        return _rootNode;
    }
}

